// NpSession.h: interface for the CNpSession class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "INpSession.h"
#include "IckarSocket.h"

#include "NpPacket.h"

typedef enum _E_PACKET_PROCESS
{
	ePP_READY,
	ePP_WAIT_HEADER_END,
	ePP_WAIT_BODY_END
} E_PACKET_PROCESS;

#pragma warning(disable:4786)
#include <list>
using namespace std;


/**
  @ingroup NpSessionBase
  @brief INpSession ̽ Ͽ Ʈ  ߻ȭ Ŭ. IIcarSocket κ ̺Ʈ Ѵ.
  @author 翵
*/
class CNpSession : public INpSession,
	public _IIckarSocketEvents
{
public:
	CNpSession();
	virtual ~CNpSession();

	IIckarSocket * m_pIckarSocket;

	bool m_bUseProxySetting;

	///////////////////////////////////////////////////////////////
	// INpSession interface methods
	virtual void SetEvents(_INpSessionEvents * pEvents);

	virtual IIckarSocket * GetIckarSocket();
	virtual bool AttachIckarSocket(IIckarSocket * pIckarSocket);
	virtual IIckarSocket * DetachIckarSocket();

	virtual bool ConnectSession(LPCSTR szHostAddr, int nHostPort, bool bUseProxySetting = true, int nTimeout = -1);

	virtual void CloseSession();

	virtual void SendRawData(LPSTR pBuffer, int nDataLen);
	virtual int RecvRawData(LPSTR pBuffer, int nDataLen);
	
	virtual bool SendNpPacket(const INpPacket * pPacket);
	virtual bool SendNpPacket_M6V2(USHORT cmmd, ULONG trid, USHORT usBodyDataLength, LPCSTR szBodyData, LPC_NP_UTF8 pu8_To = NULL, LPC_NP_UTF8 pu8_From = NULL, USHORT usSessionId = 0);
	virtual bool SendNpPacket_M6V2(const M6V2_HEADER & header, USHORT usBodyDataLength, LPCSTR szBodyData, LPC_NP_UTF8 pu8_To = NULL, LPC_NP_UTF8 pu8_From = NULL);
	virtual bool SendNpPacket_MediaData(USHORT trid, USHORT usBodyDataLength, LPCSTR szBodyData);
	virtual bool SendNpPacket_MediaPing(USHORT trid);
	virtual bool SendNpPacket_SessionControlPacket(UCHAR data, bool checkbit);

	virtual void Delete();

	virtual bool SendNpPacket_M6V2(USHORT cmmd, ULONG trid, USHORT usBodyDataLength, LPCSTR szBodyData, BYTE ucEncType, LPC_NP_UTF8 pu8_To = NULL, LPC_NP_UTF8 pu8_From = NULL, USHORT usSessionId = 0);

protected:
	_INpSessionEvents * m_pEvents;
	
	void fire_OnConnect(int nSocketError);
	void fire_OnClose(int nSocketError, bool bRecvInvalidPacket);
	void fire_OnRecvPacket();
	void fire_OnSendReady();
	int	 fire_OnRecvRawData(LPCSTR szRawData, ULONG ulDataLength);

	bool Check_AsyncDelete();
#if defined(WIN32)
	volatile LONG m_nFiringEventCount;
#else
	int  m_nFiringEventCount;	
#endif
	bool m_bDeleteMark;

	int ParseSessionStream( char * pszRawData, int nSize );

	E_PACKET_PROCESS m_ePacketProcess;
	CNpPacket * m_pNcCurrentPacket;

	char * m_pReceiveBuffer;
	int m_nReceiveBufferSize;
	int m_nCurrentReceiveLength;

	bool InitIckarSocket();

	///////////////////////////////////////////////////////////////
	// receive events : _IIckarSocketEvents
	virtual void CALLBACK OnIckarSocket_Connect(IIckarSocket * pIckarSocket, int nSocketError);
	virtual void CALLBACK OnIckarSocket_Close(IIckarSocket * pIckarSocket, int nSocketError);

	virtual void CALLBACK OnIckarSocket_Accept(IIckarSocket * pIckarSocket, int nSocketError);

	virtual void CALLBACK OnIckarSocket_SendReady(IIckarSocket * pIckarSocket);

	virtual void CALLBACK OnIckarSocket_ReceiveReady(IIckarSocket * pIckarSocket, int nBytes);
};


inline
void CNpSession::fire_OnConnect(int nSocketError)
{
	InterlockedIncrement(&m_nFiringEventCount);
	if(m_pEvents)
	{
		m_pEvents->OnNpSession_Connect(this, nSocketError);
	}
	InterlockedDecrement(&m_nFiringEventCount);
}

inline
void CNpSession::fire_OnClose(int nSocketError, bool bRecvInvalidPacket)
{
	InterlockedIncrement(&m_nFiringEventCount);
	if(m_pEvents)
	{
		m_pEvents->OnNpSession_Close(this, nSocketError, bRecvInvalidPacket);
	}
	InterlockedDecrement(&m_nFiringEventCount);
}

inline
void CNpSession::fire_OnRecvPacket()
{
	InterlockedIncrement(&m_nFiringEventCount);
	if(m_pEvents)
	{
		m_pEvents->OnNpSession_RecvPacket(this, m_pNcCurrentPacket);
	}
	InterlockedDecrement(&m_nFiringEventCount);
}

inline
void CNpSession::fire_OnSendReady()
{
	InterlockedIncrement(&m_nFiringEventCount);
	if(m_pEvents)
	{
		m_pEvents->OnNpSession_SendReady(this);
	}
	InterlockedDecrement(&m_nFiringEventCount);
}

inline
int	 CNpSession::fire_OnRecvRawData(LPCSTR szRawData, ULONG ulDataLength)
{
	InterlockedIncrement(&m_nFiringEventCount);

	int nRet = 0;
	if(m_pEvents)
	{
		nRet = m_pEvents->OnNpSession_RecvRawData(this, szRawData, ulDataLength);
	}
	
	InterlockedDecrement(&m_nFiringEventCount);

	return nRet;
}

inline
bool CNpSession::Check_AsyncDelete()
{
	if( m_bDeleteMark )
	{
		if( m_nFiringEventCount == 0 )
		{
			delete this;
			return true;
		}
	}

	return false;
}
